unit main;

{
HISTORY
 2-16-2007  1.0  First released
 2-20-2007  1.1  Handles filenames with special chars ("+", "{", etc.).  Added abort button, date&time display
 7-02-2007  1.2  Handles streams with a single audio program
 7-04-2007  1.3  /LOGONLY option
 2-02-2008  1.4  /AT option to select audio tracks other than #1
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;

type
  TfrmMain = class(TForm)
    Memo1: TMemo;
    Panel1: TPanel;
    Timer1: TTimer;
    Button1: TButton;
    procedure FormActivate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation
{$R *.DFM}
uses DOS, SndKey32;

var
  sIn, sOut: string;
  bSAT, bLO: boolean;
  ATrk: integer;

procedure pause(tenths_of_a_second: integer);
  var i: integer;
begin
  frmMain.Panel1.Caption := DateToStr(Date) + '  ' + TimeToStr(Time);
  application.ProcessMessages;
  for i := 1 to tenths_of_a_second do
  begin
    if (frmMain.Tag = 0) then
      sleep(100); //skip pause if we're gonna bail
    application.ProcessMessages;
  end;
end;

procedure log(s: string);
  var LogFile : TextFile;
    FileName : string;
begin
  frmMain.Memo1.text := frmMain.Memo1.text + TimeToStr(Time) + '   ' + s + chr(13) + chr(10);
  pause(0);

  FileName := ChangeFileExt(Application.EXEName,'.LOG');
  AssignFile(LogFile,FileName);
  if FileExists(FileName) then
  begin
    try
      Append(LogFile);
    except
      //for some reason, sometimes there's not enough of a delay after the
      //last close, and Windows thinks the file's still open, causing
      //Delphi to return an "i/o error 32" message.
      //Waiting and trying again seems to take care of the problem.
      sleep(500);
      Append(LogFile);
    end;
  end Else
  begin
    ReWrite(LogFile);
  end;
  Write(LogFile,DateToStr(Date) + '  ');
  Write(LogFile,TimeToStr(Time) + ' - ');
  WriteLn(LogFile,s);
  Flush(LogFile);
  CloseFile(LogFile);
end;

procedure usage;
  var s: string;
begin
  s := 'mpeg2repairHelper - allows you to run mpeg2repair from the command line.'+chr(13)+chr(13);
  s := s+'Version 1.3 by Charles Boling'+chr(13)+chr(13);
  s := s+'Usage:  mpeg2repairHelper  [/SAT]|[/AT n]  [/LOGONLY]  inputfile  outputfile'+chr(13)+chr(13);;
  s := s+'              (full paths recommended)'+chr(13)+chr(13);
  s := s+'Options:'+chr(13);
  s := s+'     /SAT:          Must be specified when only a single audio track is present in source file.'+chr(13);
  s := s+'     /LOGONLY: Don''t produce an output file.'+chr(13);
  s := s+'     /AT {n}:      Extract Audio track #{n} instead of track #1'+chr(13);
  s := s+''+chr(13);
  s := s+'Notes:'+chr(13);
  s := s+'     If log file exists, it will be overwritten.'+chr(13);
  s := s+'     If video output file exists, the program will abort.'+chr(13);
  showmessage(s);
  Application.Terminate;
end;

function CmdLineOpt(sOpt: string): integer;
// Searches for command-line option and Returns parameter index (or 0 if not found)
  var
    i: integer;
begin
  result := 0;
  for i := 1 to ParamCount do
  begin
    if comparetext('/'+sOpt,ParamStr(i))=0 then
      result := i;
  end;
end;

function SafeStrToInt(s : string): integer;
  //returns -1 if non-integer
begin
  try
    Result := StrToInt(s);
  except
    on E: EConvertError do
    begin
      Result := -1;
    end;
  end;
end;

procedure GetParams;
  var
    i: integer;
begin
  i := 0;
  bSAT := FALSE;
  bLO := FALSE;

  if CmdLineOpt('SAT') > 0 then
  begin
    bSAT := TRUE;
    inc(i);
  end;
  if CmdLineOpt('LogOnly') > 0 then
  begin
    bLO := TRUE;
    inc(i);
  end;
  ATrk := 1;
  if CmdLineOpt('AT') > 0 then
  begin
    inc(i);
    inc(i);
    ATrk := SafeStrToInt(ParamStr(CmdLineOpt('AT')+1));
    if ATrk < 1 then
    begin
      ShowMessage('Invalid audio track specified: '+ParamStr(CmdLineOpt('AT')+1));
      sOut := '';
      Application.Terminate;
    end;
  end;
  If (ParamCount = i+2) then
  begin
    sIn := ParamStr(i+1);
    sOut := ParamStr(i+2);
    if not FileExists(sIn) then
    begin
      ShowMessage('Input file:'+chr(13)+'  '+sIn+chr(13)+'doesn''t exist.');
      sOut := '';
      Application.Terminate;
    end;
    if FileExists(sOut) then
    begin
      ShowMessage('Output file:'+chr(13)+'  '+sOut+chr(13)+'already exists.');
      sOut := '';
      Application.Terminate;
    end;
    if FileExists(sOut+'.txt') then
    begin
      ShowMessage('Error Log:'+chr(13)+'  '+sOut+'.txt'+chr(13)+'already exists.');
      sOut := '';
      Application.Terminate;
    end;
  end else
  begin
    usage;
  end;
end;

procedure start_mpeg2repair;
  var
    ExitCode: longword;
    i: integer;
begin
  If AppActivate('mpeg2repair') then
  begin
    AppActivate('mpeg2repairHelper');
    ShowMessage('mpeg2repair window already open.  All instances must be closed before starting.');
    Application.Terminate;
    exit;
  end;
  if AppActivate('Status:') then
  begin
    AppActivate('mpeg2repairHelper');
    ShowMessage('A window titled "Status:" is already open.  All instances must be closed before starting.');
    Application.Terminate;
    exit;
  end;
  If DosCmd('mpeg2repair.exe','',false,ExitCode) then
  begin
    pause(12); //2-2-08 was 5
    if AppActivate('MPEG2Repair') then
    begin
      pause(1);
      SendKeys(pchar(CleanSendKeysString(sIn)),true); // Input file
      pause(5);
      SendKeys(pchar('{TAB}{TAB}'),true); // Now on Find PID's
      pause(5);
      log('  Finding PIDs');
      SendKeys(pchar('( )'),true); // Find PID's
      pause(50);
      If not bSAT then
      begin
        SendKeys(pchar('{TAB}'),true); //Now on Audio Track
        if ATrk > 1 then
        begin
          for i := 2 to ATrk do
          begin
            SendKeys(pchar('{Down}'),true); //Inc Track
            pause(1);
            SendKeys(pchar('%{TAB}'),true); //Switch focus away from app
            pause(7);
            SendKeys(pchar('%{TAB}'),true); //Switch Back
            pause(5);
            SendKeys(pchar('{TAB}{TAB}'),true); // Now on Find PID's
            pause(6);
            if i = ATrk then
            begin
              SendKeys(pchar('( )'),true); // Find PID's
              pause(50);
            end;
            SendKeys(pchar('{TAB}'),true); //Now on Audio Track (again)
            pause(5);
          end;
        end;
        pause(3);
        SendKeys(pchar('{TAB}{TAB}{TAB}'),true); //Now on Repair Errors
        pause(3);
      end;
      log('  Filling out the rest of the form');
      If bLO then
      begin
        log('Option: LogOnly');
        SendKeys(pchar('{TAB}'),true);
      end else
      begin
        log('Option: Repair');
        SendKeys(pchar('( )'),true); //Repair
        pause(10);
        log('Specify output file...');
        SendKeys(pchar('{TAB}'+CleanSendKeysString(sOut)),true); //Output file
        pause(3);
        log('...done');
        SendKeys(pchar('{TAB}{TAB}'),true);
      end;
      log('Activate Logging');
      SendKeys(pchar('( )'),true); //Log Errors
      pause(10);
      SendKeys(pchar('{TAB}'+sOut+'.txt'),true); //Log File
      pause(2);
      SendKeys(pchar('{TAB}{TAB}'),true);  //Ready?
      pause(2);
      SendKeys(pchar('{Enter}'),true);  //Start!
      pause(1);
    end else
    begin
      ShowMessage('Failed to activate mpeg2repair.exe');
      Application.Terminate;
    end;
  end else
  begin
    ShowMessage('mpeg2repair.exe and mpeg2repairHelper'+chr(13)+'must be in the same directory.');
    Application.Terminate;
  end;
end;

procedure monitor_mpeg2repair;
begin
  repeat
    pause(10)
  until AppActivate('Status:') or (frmMain.Tag > 0);
  if frmMain.Tag = 0 then
  begin
    log('mpeg2repair appears to be finished.');
    SendKeys(pchar('{Enter}'),true);  //Okay
    pause(3);
    AppActivate('mpeg2repair');
    SendKeys(pchar('%{F4}'),true);  //Exit app
    pause(20);
    if AppActivate('mpeg2repair') then
      log('failed to terminate mpeg2repair within 2 seconds.')
    else
      log('mpeg2repair exited.');
  end;
end;

procedure DoStuff;
begin
  frmMain.Memo1.Text := '';
  sOut := '';
  log('Program Started');
  GetParams;
  if sOut > '' then
  begin
    log('Starting mpeg2repair');
    start_mpeg2repair;
    log('Monitoring mpeg2repair');
    monitor_mpeg2repair;
  end;
  log('Program Ended');
  frmMain.Close;
end;

procedure TfrmMain.FormActivate(Sender: TObject);
begin
  Timer1.Enabled := TRUE;
end;

procedure TfrmMain.Timer1Timer(Sender: TObject);
begin
  Timer1.Enabled := False;
  DoStuff;
end;

procedure TfrmMain.Button1Click(Sender: TObject);
begin
  Beep;
  Log('Aborted by user');
  frmMain.Tag := 1; //flag abort
end;

end.
